@@ -245,7 +245,11 @@ void embedcmdfree(embedcmd_T c)
 
 /********** Auxiliary Functions for Parser **********/
 
-static wchar_t *skip_name(const wchar_t *s)
+static wchar_t *skip_name(const wchar_t *s, bool predicate(wchar_t))
+    __attribute__((pure,nonnull));
+static bool is_name_by_predicate(const wchar_t *s, bool predicate(wchar_t))
+    __attribute__((pure,nonnull));
+static bool is_portable_name(const wchar_t *s)
     __attribute__((pure,nonnull));
 
 
@@ -281,19 +285,33 @@ bool is_name_char(wchar_t c)
 
 /* Skips an identifier at the head of the specified string and returns a
  * pointer to the character right after the identifier in the string.
- * If there is no identifier, the argument `s' is simply returned. */
-wchar_t *skip_name(const wchar_t *s)
+ * If there is no identifier, the argument `s' is simply returned. `predicate`
+ * determines if a character is valid. */
+wchar_t *skip_name(const wchar_t *s, bool predicate(wchar_t))
 {
     if (!iswdigit(*s))
-	while (is_name_char(*s))
+	while (predicate(*s))
 	    s++;
     return (wchar_t *) s;
 }
 
+/* Returns true iff the specified string constitutes a valid identifier that
+ * is made up of characters accepted by `predicate'. */
+bool is_name_by_predicate(const wchar_t *s, bool predicate(wchar_t))
+{
+    return s[0] != L'\0' && skip_name(s, predicate)[0] == L'\0';
+}
+
+/* Returns true iff the specified string constitutes a valid portable name. */
+bool is_portable_name(const wchar_t *s)
+{
+    return is_name_by_predicate(s, is_portable_name_char);
+}
+
 /* Returns true iff the specified string constitutes a valid identifier. */
 bool is_name(const wchar_t *s)
 {
-    return s[0] != L'\0' && skip_name(s)[0] == L'\0';
+    return is_name_by_predicate(s, is_name_char);
 }
 
 /* Returns true iff the string is a reserved word. */
@@ -2161,7 +2179,7 @@ command_T *parse_for(parsestate_T *ps)
     result->c_redirs = NULL;
 
     wchar_t *name = parse_word_as_wcs(ps);
-    if (!is_name(name)) {
+    if (!(posixly_correct ? is_portable_name : is_name)(name)) {
 	if (name[0] == L'\0')
 	    serror(ps, Ngt("an identifier is required after `for'"));
 	else
@@ -2330,11 +2348,14 @@ void **parse_case_patterns(parsestate_T *ps)
 		    "an unquoted `esac' cannot be the first case pattern"));
 	}
     }
+
+    const wchar_t *predecessor = L"(";
     do {
 	if (is_token_delimiter_char(ps->src.contents[ps->index])) {
 	    if (ps->src.contents[ps->index] != L'\0') {
 		if (ps->src.contents[ps->index] == L'\n')
-		    serror(ps, Ngt("a word is required after `%ls'"), L"(");
+		    serror(ps, Ngt("a word is required after `%ls'"),
+			    predecessor);
 		else
 		    serror(ps, Ngt("encountered an invalid character `%lc' "
 				"in the case pattern"),
@@ -2346,6 +2367,7 @@ void **parse_case_patterns(parsestate_T *ps)
 	skip_blanks_and_comment(ps);
 	ensure_buffer(ps, 1);
 	if (ps->src.contents[ps->index] == L'|') {
+	    predecessor = L"|";
 	    ps->index++;
 	} else if (ps->src.contents[ps->index] == L')') {
 	    ps->index++;
@@ -2415,7 +2437,8 @@ command_T *tryparse_function(parsestate_T *ps)
     if (iswdigit(ps->src.contents[ps->index]))
 	goto fail;
 
-    size_t namelen = count_name_length(ps, is_name_char);
+    size_t namelen = count_name_length(ps,
+	    posixly_correct ? is_portable_name_char : is_name_char);
     ps->index += namelen;
     if (namelen == 0 || !is_token_delimiter_char(ps->src.contents[ps->index]))
 	goto fail;
@@ -3198,8 +3221,11 @@ void print_word(struct print *restrict pr, const wordunit_T *restrict wu,
 		break;
 	    case WT_CMDSUB:
 		wb_cat(&pr->buffer, L"$(");
+		size_t startindex = pr->buffer.length;
 		print_embedded_command(pr, wu->wu_cmdsub, indent + 1);
 		wb_wccat(&pr->buffer, L')');
+		if (pr->buffer.contents[startindex] == L'(')
+		    wb_insert(&pr->buffer, startindex, L" ");
 		break;
 	    case WT_ARITH:
 		wb_cat(&pr->buffer, L"$((");
